package src

import (
	"crypto"
	"crypto/ecdsa"
	"crypto/elliptic"
	"crypto/rand"
	"crypto/x509"
	"encoding/json"
	"github.com/go-acme/lego/v4/certificate"
	"github.com/go-acme/lego/v4/lego"
	"github.com/go-acme/lego/v4/log"
	"github.com/go-acme/lego/v4/providers/dns/alidns"
	"github.com/go-acme/lego/v4/registration"
	"os"
	"path"
	"path/filepath"
)

// Apply 申请证书
// domain: 申请的域名
func Apply(domain Domain) {
	email, hosts, name := domain.Email, domain.Host, domain.Name
	privateKey, err := ecdsa.GenerateKey(elliptic.P256(), rand.Reader)
	if err != nil {
		log.Fatal(err)
	}

	acmeUser := AcmeUser{
		Email: email,
		key:   privateKey,
	}

	config := lego.NewConfig(&acmeUser)

	client, err := lego.NewClient(config)
	if err != nil {
		log.Fatal(err)
	}

	ali := appConfig.Provider.Ali
	conf := alidns.NewDefaultConfig()
	conf.RegionID = ali.RegionID
	conf.APIKey = ali.APIKey
	conf.SecretKey = ali.SecretKey
	provider, err := alidns.NewDNSProviderConfig(conf)
	if err != nil {
		log.Fatal(err)
	}

	challenge := client.Challenge
	err = challenge.SetDNS01Provider(provider)
	if err != nil {
		return
	}

	registrar := client.Registration
	reg, err := registrar.Register(registration.RegisterOptions{TermsOfServiceAgreed: true})
	if err != nil {
		log.Fatal(err)
	}
	acmeUser.Registration = reg

	request := certificate.ObtainRequest{
		Domains: hosts,
		Bundle:  true,
	}

	cert, err := client.Certificate.Obtain(request)
	if err != nil {
		log.Fatal(err)
	}

	//tlsCert, err := tls.X509KeyPair(cert.Certificate, cert.PrivateKey)
	//if err != nil {
	//	log.Fatal(err)
	//}
	//log.Printf("Certificate: %+v", tlsCert)
	//log.Printf("Certificate: %+v", cert)
	//log.Printf("Certificate: %+v", cert.Domain)
	//log.Printf("Certificate: %+v", cert.CertURL)

	saveCertFile(cert, name)
}

// saveCertFile 保存证书文件
func saveCertFile(cert *certificate.Resource, name string) {
	dir := GetAppConfig().CertDir
	dir = filepath.Join(dir, name)
	_, err := os.Stat(dir)
	if os.IsNotExist(err) {
		err := os.MkdirAll(dir, 0644)
		if err != nil {
			log.Infof("创建目录 %s 失败", dir)
			log.Fatal(err)
		}
		log.Infof("创建目录 %s", dir)
	}

	err = os.WriteFile(path.Join(dir, "cert.crt"), cert.Certificate, 0644)
	if err != nil {
		log.Fatalf("Failed to save certificate: %v", err)
	}

	err = os.WriteFile(path.Join(dir, "cert.key"), cert.PrivateKey, 0644)
	if err != nil {
		log.Fatalf("Failed to save private key: %v", err)
	}

	// 获取 cert 的过期时间
	certParse, err := x509.ParseCertificate(cert.Certificate)
	if err != nil {
		log.Fatalf("Failed to parse certificate: %v", err)
	}
	certInfo := CertInfo{
		Cert: *cert,
		Info: *certParse,
	}
	// 将 cert 转换为json格式并保存到文件中
	certJson, err := json.Marshal(certInfo)
	if err != nil {
		log.Fatalf("Failed to marshal certificate: %v", err)
	}
	err = os.WriteFile(path.Join(dir, "info.json"), certJson, 0644)
	if err != nil {
		log.Fatalf("Failed to save certificate info: %v", err)
	}

}

type AcmeUser struct {
	Email        string
	Registration *registration.Resource
	key          crypto.PrivateKey
}

func (u AcmeUser) GetEmail() string {
	return u.Email
}
func (u AcmeUser) GetRegistration() *registration.Resource {
	return u.Registration
}
func (u AcmeUser) GetPrivateKey() crypto.PrivateKey {
	return u.key
}

type CertInfo struct {
	Cert certificate.Resource
	Info x509.Certificate
}
